@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
 *
-*   Copyright (C) 1999-2001, International Business Machines
+*   Copyright (C) 1999-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************
@@ -141,72 +141,6 @@ setCommonICUData(UDataMemory *pData,     /*  The new common data.  Belongs to ca
     return;
 }
 
-
-
-static char *
-strcpy_returnEnd(char *dest, const char *src) {
-    while((*dest=*src)!=0) {
-        ++dest;
-        ++src;
-    }
-    return dest;
-}
-
-/*------------------------------------------------------------------------------
- *                                                                              
- *  computeDirPath   given a user-supplied path of an item to be opened,             
- *                         compute and return 
- *                            - the full directory path to be used 
- *                              when opening the file.
- *                            - Pointer to null at end of above returned path    
- *
- *                       Parameters:
- *                          path:        input path.  Buffer is not altered.
- *                          pathBuffer:  Output buffer.  Any contents are overwritten.
- *
- *                       Returns:
- *                          Pointer to null termination in returned pathBuffer.
- *
- *                    TODO:  This works the way ICU historically has, but the
- *                           whole data fallback search path is so complicated that
- *                           proabably almost no one will ever really understand it,
- *                           the potential for confusion is large.  (It's not just 
- *                           this one function, but the whole scheme.)
- *                            
- *------------------------------------------------------------------------------*/
-char *
-uprv_computeDirPath(const char *path, char *pathBuffer) {
-    char   *finalSlash;       /* Ptr to last dir separator in input path, or null if none. */
-    int32_t pathLen;          /* Length of the returned directory path                     */
-    
-    finalSlash = 0;
-    if (path != 0) {
-        finalSlash = uprv_strrchr(path, U_FILE_SEP_CHAR);
-    }
-    
-    *pathBuffer = 0;
-    if (finalSlash == 0) {
-        /* No user-supplied path.  
-         * Copy the ICU_DATA path to the path buffer and return that*/
-        const char *icuDataDir;
-        icuDataDir=u_getDataDirectory();
-        if(icuDataDir!=NULL && *icuDataDir!=0) {
-            return strcpy_returnEnd(pathBuffer, icuDataDir);
-        } else {
-            /* there is no icuDataDir either.  Just return the empty pathBuffer. */
-            return pathBuffer;
-        }
-    } 
-    
-    /* User supplied path did contain a directory portion.
-     * Copy it to the output path buffer */
-    pathLen = (int32_t)(finalSlash - path + 1);
-    uprv_memcpy(pathBuffer, path, pathLen);
-    *(pathBuffer+pathLen) = 0;
-    return pathBuffer+pathLen;
-}
-
-
 static const char *
 findBasename(const char *path) {
     const char *basename=uprv_strrchr(path, U_FILE_SEP_CHAR);
@@ -537,7 +471,8 @@ static const char *udata_pathiter_next(UDataPathIterator *iter, int32_t *outPath
         /* check for .dat files */
         pathBasename = findBasename(iter->pathBuffer);
 
-        if(iter->checkLastFour == TRUE &&
+        if(iter->checkLastFour == TRUE && 
+           (pathLen>=4) &&
            uprv_strncmp(iter->pathBuffer +(pathLen-4),iter->suffix,4)==0 && /* suffix matches */
            uprv_strncmp(findBasename(iter->pathBuffer),iter->basename,iter->basenameLen)==0  && /* base matches */
            uprv_strlen(pathBasename)==(iter->basenameLen+4)) { /* base+suffix = full len */
@@ -551,8 +486,17 @@ static const char *udata_pathiter_next(UDataPathIterator *iter, int32_t *outPath
         {       /* regular dir path */
             if(iter->pathBuffer[pathLen-1] != U_FILE_SEP_CHAR)  /* trailing sep */
             {
+                if((pathLen>=4) &&
+                   uprv_strncmp(iter->pathBuffer+(pathLen-4), ".dat", 4) == 0)
+                {
+#ifdef UDATA_DEBUG
+                    fprintf(stderr, "skipping non-directory .dat file %s\n", iter->pathBuffer);
+#endif
+                    continue;
+                }
+                     
                 iter->pathBuffer[pathLen++] = U_FILE_SEP_CHAR;
-            }
+            } 
             
             uprv_strncpy(iter->pathBuffer + pathLen,  /* + basename */
                          iter->basename,
@@ -593,7 +537,7 @@ static const char *udata_pathiter_next(UDataPathIterator *iter, int32_t *outPath
 /*----------------------------------------------------------------------*
  *                                                                      *
  *  Add a static reference to the common data  library                  *
- *   Unless overridden by an explicit u_setCommonData, this will be     *
+ *   Unless overridden by an explicit udata_setCommonData, this will be *
  *      our common data.                                                *
  *                                                                      *
  *----------------------------------------------------------------------*/
@@ -717,7 +661,7 @@ openCommonData(const char *path,          /*  Path from OpenCHoice?          */
 
 
 #ifdef OS390
-#   define MAX_STUB_ENTRIES 7
+#   define MAX_STUB_ENTRIES 8
 #else
 #   define MAX_STUB_ENTRIES 0
 #endif
@@ -805,7 +749,7 @@ udata_setCommonData(const void *data, UErrorCode *pErrorCode) {
 
     /* do we already have common ICU data set? */
     if(gCommonICUData != NULL) {
-        *pErrorCode=U_USING_DEFAULT_ERROR;
+        *pErrorCode=U_USING_DEFAULT_WARNING;
         return;
     }
 
@@ -978,7 +922,6 @@ doOpenChoice(const char *path, const char *type, const char *name,
 
     /* the data was not found in the common data,  look further, */
     /* try to get an individual data file */
-    /* === basename=uprv_computeDirPath(path, pathBuffer); === */
     if(path == NULL) {
         path = COMMON_DATA_NAME;
         inBasename = COMMON_DATA_NAME;
@@ -1103,7 +1046,7 @@ doOpenChoice(const char *path, const char *type, const char *name,
         if (!(isICUData && pCommonData && extendICUData(pCommonData, &errorCode))) {
             break;
         }
-    };
+    }
 
     /* data not found */
     if(U_SUCCESS(*pErrorCode)) {
